# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pygt3x']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.21.2,<2.0.0', 'pandas>=1.0.5,<2.0.0']

setup_kwargs = {
    'name': 'pygt3x',
    'version': '0.1.3',
    'description': 'Python module for reading GT3X/AGDC file format data',
    'long_description': '# pygt3x\n![Tests](https://github.com/actigraph/pygt3x/actions/workflows/tests.yml/badge.svg)\n\nPython module for reading GT3X/AGDC file format data generated by ActiGraph devices.\n\n## Example Usage\n\n```python\nfrom pygt3x.reader import FileReader\nfrom pygt3x.calibrated_reader import CalibratedReader\n\n# Read raw data and calibrate\n# Dump to pandas data frame\nwith FileReader("WRIST_rawLSB_032Hz.agdc") as reader:\n    calibrated_reader =  CalibratedReader(reader)\n    df = calibrated_reader.to_pandas()\n    print(df.head(5))\n```\n',
    'author': 'Mark Fogle',
    'author_email': 'mark.fogle@theactigraph.com',
    'maintainer': 'Ali Neishabouri',
    'maintainer_email': 'ali.neishabouri@theactigraph.com',
    'url': 'https://github.com/actigraph/pygt3x',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
